.grid > .toggle {
  transition: transform 200ms ease-out, background 200ms ease-out;
  border-radius: 12px;
  transform: scale(0.9);
}

.grid > .toggle:checked/*, .grid > .toggle:hoverr */{
  background-image: linear-gradient(
    45deg,
    oklch(from var(--accent-bg-color) calc(l - 0.1) c calc(h + 10deg)),
    oklch(from var(--accent-bg-color) calc(l + 0.1) c calc(h - 10deg))
  );
  transform: none;
}

.grid {
  margin: 12px;
  margin-top: 0;
}
